package com.common.app.base;

import android.app.Activity;
import android.content.Context;
import android.databinding.DataBindingUtil;
import android.databinding.ViewDataBinding;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.common.app.R;
import com.common.app.databinding.BaseViewBinding;
import com.common.app.ui.util.LoadingDialogUtil;


/**
 * Created by chenhongfei on 2017/4/14.
 */

public abstract class BaseDataBindingFragment<DBinding extends ViewDataBinding> extends Fragment implements IBaseViewShow {
    private BaseViewBinding mBaseViewBinding;
    private DBinding mContentViewBinding;
    private ViewDataBinding mTopbarViewBinding;

    protected String mEmptyTip;
    protected boolean mHasTopbar = true;
    protected Activity mActivity;
    private View mRootView;
    //mIsOpenLazy为false的时候忽略
    protected boolean mIsLoaded;
    protected boolean mIsOpenLazy;
    protected boolean mIsViewCreated;
    protected boolean mIsVisible;

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        this.mActivity = getActivity();
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        initParams(savedInstanceState);


        mBaseViewBinding = DataBindingUtil.inflate(inflater, R.layout.base_view, container, false);
        mContentViewBinding = DataBindingUtil.inflate(inflater, getContentViewId(), mBaseViewBinding.rlContainer, true);
        if (getTopbarViewId() > 0 && mHasTopbar) {
            mTopbarViewBinding = DataBindingUtil.inflate(inflater, getTopbarViewId(), mBaseViewBinding.flTitle, true);
        }

        mBaseViewBinding.llNetworkError.llNetworkInvalid.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                retryLoadData();
            }
        });


        mRootView = mBaseViewBinding.getRoot();
        mIsViewCreated = true;
        setEmptyTip();
        return mRootView;
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        initView(mRootView);
        doLoadData();
    }

    @Override
    public void setUserVisibleHint(boolean isVisibleToUser) {
        super.setUserVisibleHint(isVisibleToUser);
        mIsVisible = isVisibleToUser;
        if (isVisibleToUser && mContentViewBinding != null) {
            doLoadData();
        }
    }

    public BaseViewBinding getRootViewBinding() {
        return mBaseViewBinding;
    }

    public View getEmptyView() {
        return mBaseViewBinding.llEmptyContent.llEmptyContent;
    }

    public DBinding getContentViewBinding() {
        return mContentViewBinding;
    }

    public ViewDataBinding getTopbarViewBinding() {
        return mTopbarViewBinding;
    }

    public View getRootView() {
        return mRootView;
    }

    private void doLoadData() {
        if (mIsOpenLazy) {
            lazyLoadData();
        } else {
            loadData();
        }
    }

    /**
     * 通过setUserVisibleHint来实现网络懒加载
     * setUserVisibleHint不一定会被系统调用，page中有调用，其他地方要自己主动调用
     * 需要在网络获取失败处重置mIsLoaded = false
     */
    private void lazyLoadData() {
        if (!mIsLoaded && mIsVisible) {
            loadData();
            mIsLoaded = true;
        }
    }

    protected void setEmptyTip() {
        if (TextUtils.isEmpty(mEmptyTip)) {
            mEmptyTip = this.getResources().getString(R.string.no_data);
        }
        mBaseViewBinding.setEmptyTip(mEmptyTip);
    }

    protected abstract int getContentViewId();

    protected abstract int getTopbarViewId();

    protected abstract void initParams(Bundle savedInstanceState);

    protected abstract void initView(View view);

    protected abstract void loadData();

    protected void retryLoadData() {
        loadData();
    }

    public void showLoadingView(int loadingTip) {
        LoadingDialogUtil.showLoadingDialog(mActivity, loadingTip);
    }

    @Override
    public void showLoadingView(String loadingTip) {
        LoadingDialogUtil.showLoadingDialog(mActivity, loadingTip);
    }

    @Override
    public void showErrorView() {
        closeLoadingView();
        mIsLoaded = false;
        mBaseViewBinding.llEmptyContent.llEmptyContent.setVisibility(View.GONE);
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setVisibility(View.VISIBLE);
    }

    @Override
    public void showEmptyView() {
        closeLoadingView();
        mBaseViewBinding.llEmptyContent.llEmptyContent.setVisibility(View.VISIBLE);
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setVisibility(View.GONE);
    }

    @Override
    public void showContentView() {
        closeLoadingView();
        mBaseViewBinding.llEmptyContent.llEmptyContent.setVisibility(View.GONE);
        mBaseViewBinding.llNetworkError.llNetworkInvalid.setVisibility(View.GONE);
        mBaseViewBinding.rlContainer.setVisibility(View.VISIBLE);
    }

    @Override
    public void closeLoadingView() {
        LoadingDialogUtil.closeLoadingDialog();
    }

    @Override
    public void onPause() {
        super.onPause();
//        AspLog.e("umeng", " onPageEnd:" + getClass().getName());
//        MobclickAgent.onPageEnd(this.getClass().getName());
    }

    @Override
    public void onResume() {
        super.onResume();
//        MobclickAgent.onPageStart(this.getClass().getName());
    }
}
