package com.common.app.data.base;

import android.content.Context;
import android.text.TextUtils;

import com.common.app.base.BaseApplication;
import com.common.framework.util.AspLog;
import com.common.framework.util.DatetimeUtil;
import com.common.framework.util.FileUtils;
import com.google.gson.Gson;

import java.io.File;


/**
 * Created by chenhongfei on 2016/11/17.
 */
public abstract class BaseCache<T> implements ICache<T> {
    private static final String TAG = "BaseCache";

    protected Context mContext = BaseApplication.getInstance().getApplicationContext();
    protected static Gson mGson = new Gson();

    protected T mResult;
    protected JsonCacheBean<T> mJsonCacheBean = null;

    /**
     * 加载数据的状态机
     */
    protected static final int LOAD_DATA_STATE_NOT_INIT = 0;
    protected static final int LOAD_DATA_STATE_ING = 1;
    protected static final int LOAD_DATA_STATE_FINISH = 2;
    // 数据是否被刷新过，不能用stattic修饰
    protected int mDataRefreshState = LOAD_DATA_STATE_NOT_INIT;

    // 有效期，默认是一天
    protected long mValidTimestamp = 86400 * 1000;

    public T getData() {
        AspLog.d(TAG, "T:" + this.getClass().toString() + " mDataRefreshState:" + mDataRefreshState);
        if (!isCacheExpired()) {
            if (mDataRefreshState == LOAD_DATA_STATE_NOT_INIT) {
                initData();
            }
            return mResult;
        }
        initData();

        return null;
    }

    public abstract void initData();

    @Override
    public boolean isCacheExpired() {
        return mResult == null;
    }

    @Override
    public T getCacheData() {
        return mResult;
    }

    @Override
    public boolean saveCacheData(T info) {
        if (info == null) {
            return false;
        }
        mResult = info;

        boolean result = false;
        long lCurrTime = System.currentTimeMillis();
        long lExpiredTime = lCurrTime + getValidTimestamp();
        mJsonCacheBean = new JsonCacheBean<T>(info, lCurrTime, lExpiredTime, BaseApplication.VERSION_NAME);

        try {
            String jsonString = mGson.toJson(mJsonCacheBean);
            if (!TextUtils.isEmpty(jsonString)) {
                result = FileUtils.writeFileData(mContext, getCacheFileName(), jsonString);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    @Override
    public boolean clearCache() {
        mResult = null;
        return true;
    }

    /**
     * 文件是否过期
     *
     * @param jsonCacheBean
     * @return
     */
    public boolean isFileCacheExpired(JsonCacheBean<T> jsonCacheBean) {
        return checkFileCacheExpiredByNaturalDay(jsonCacheBean);
    }

    /**
     * 根据自然天判断是否过期
     *
     * @param jsonCacheBean
     * @return
     */
    public boolean checkFileCacheExpiredByNaturalDay(JsonCacheBean<T> jsonCacheBean) {
        try {
            if (jsonCacheBean == null) {
                return true;
            }
            long lSaveTime = jsonCacheBean.lSaveTime;
            long lExpiredTime = jsonCacheBean.lExpiredTime;
            String lVersion = jsonCacheBean.lVersion;
            long lCurrentTime = System.currentTimeMillis();
            if (lSaveTime <= 0) {
                return true;
            }
            int saveDate = Integer.valueOf(DatetimeUtil.getFormatDatetime(lSaveTime, "yyyyMMdd"));
            int currDate = Integer.valueOf(DatetimeUtil.getFormatDatetime(lCurrentTime, "yyyyMMdd"));
            AspLog.v(TAG, "saveDate:" + saveDate + " currDate:" + currDate);
            if (saveDate < currDate) {
                return true;
            }
            // 比较版本号
//            AspLog.e(TAG, String.format("local version:%s, now version: %s", lVersion, Def.JSON_CLIENT_VERSION_VALUE));
            if (TextUtils.isEmpty(lVersion) || !lVersion.equals(BaseApplication.VERSION_NAME)) {
                return true;
            }

            return false;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    /**
     * 根据保存的expiredTime判断是否过期
     *
     * @param jsonCacheBean
     * @return
     */
    public boolean checkFileCacheExpiredByExpiredTime(JsonCacheBean<T> jsonCacheBean) {
        try {
            if (jsonCacheBean == null) {
                return true;
            }
            long lSaveTime = jsonCacheBean.lSaveTime;
            long lExpiredTime = jsonCacheBean.lExpiredTime;
            String lVersion = jsonCacheBean.lVersion;
            long lCurrentTime = System.currentTimeMillis();
            if (lExpiredTime <= 0) {
                return true;
            }

            AspLog.v(TAG, "lExpiredTime:" + lExpiredTime + " lCurrentTime:" + lCurrentTime);
            if (lExpiredTime < lCurrentTime) {
                return true;
            }
            // 比较版本号
//            AspLog.e(TAG, String.format("local version:%s, now version: %s", lVersion, Def.JSON_CLIENT_VERSION_VALUE));
            if (TextUtils.isEmpty(lVersion) || !lVersion.equals(BaseApplication.VERSION_NAME)) {
                return true;
            }

            return false;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    protected abstract String getCacheFileName();

    public long getValidTimestamp() {
        return mValidTimestamp;
    }

    public void setValidTimestamp(long mValidTimestamp) {
        this.mValidTimestamp = mValidTimestamp;
    }

    public boolean isFileExist() {
        boolean isExist = false;
        try {
            File file = new File(mContext.getFilesDir() + "/" + getCacheFileName());
            isExist = file.exists();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return isExist;
    }
}
