package com.common.framework.network.parser;

import android.os.Message;
import android.text.TextUtils;
import android.util.Log;

import com.common.app.R;
import com.common.framework.network.volley.HttpStatus;
import com.common.framework.util.AspLog;
import com.common.framework.util.ToastUtil;
import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;


/**
 * 一个解析接口请求返回数据的类，ui.model包下的类都应该继承ModelInfo，也可以让ErrorInfo继承ModelInfo，这样改动会小点。
 * 使用方法：
 * YSParser ciParser = new YSParser(this.getActivity(), GetJobsInfo , mHandler);
 * <p/>
 * Created by chenhongfei on 2015/7/6.
 */
public abstract class BaseParser<T extends ModelInfo> {
    protected static final String TAG = "BaseParser";
    protected NetHandler<T> mHandler;
    protected int mResId;
    protected boolean mIsInBackground;
    private Class<T> modelClass;

    public BaseParser(NetHandler<T> h, Class<T> info) {
        this.modelClass = info;
        this.mHandler = h;
    }

    public BaseParser(NetHandler<T> h, Class<T> info, int resId) {
        this(h, info);
        this.mResId = resId;
    }

    public BaseParser(NetHandler<T> h, Class<T> info, boolean inBackground) {
        this(h, info);
        this.mIsInBackground = inBackground;
    }

    public BaseParser(NetHandler<T> h, Class<T> info, int resId, boolean inBackground) {
        this(h, info, resId);
        this.mIsInBackground = inBackground;
    }

    public Class<T> getModelInfoClass() {
        return modelClass;
    }

    //非系统错误，通过modelInfo == null || modelInfo.checkResult判断
    public void parseResponse(T model) {
        if (model != null && model.checkResult()) {
//            Log.v(TAG, TAG + " parseResponse success ... error_code:" + this.getErrorCode() + " resCode:" + this.mRespCode);
            Message msg = new Message();
            msg.what = R.id.doSuccess;
            msg.obj = model;
            msg.arg1 = mResId;
            mHandler.sendMessage(msg);
        } else {
//            Log.v(TAG, TAG + " parseResponse fail ... errorCode:" + this.getErrorCode() + " resCode:" + this.mRespCode);

            Message msg = new Message();
            msg.what = R.id.doError;
            msg.obj = model;
            msg.arg1 = mResId;
            mHandler.sendMessage(msg);
        }
    }

    public void parseResponse(String json) {
        if (TextUtils.isEmpty(json)) {
            ModelInfo modelInfo = new ModelInfo() {
                @Override
                public ErrorInfo getErrorInfo() {
                    return null;
                }
            };

            Message msg = new Message();
            msg.what = R.id.doError;
            msg.obj = modelInfo;
            msg.arg1 = mResId;
            mHandler.sendMessage(msg);

        } else {
            Gson mGson = new Gson();

            T model = null;
            int what = R.id.doSuccess;
            try {
                model = mGson.fromJson(json, modelClass);
            } catch (JsonSyntaxException jse) {
                what = R.id.doDataError;
            } catch (Exception e) {
                what = R.id.doDataError;
            }

            Message msg = new Message();
            msg.what = what;
            msg.obj = model;
            msg.arg1 = mResId;
            mHandler.sendMessage(msg);
        }

    }

    //系统错误
    public void parseErrorResponse(int statusCode) {
        Log.e(TAG, TAG + " parseErrorResponse fail ... resCode:" + statusCode + " isBackground:" + mIsInBackground);

        /**
         * 此状态下，ModelInfo的error是null
         *
         * 还是需要把错误通知外部，外部可能需要根据次状态进行一些特殊处理，例如，finishe activity
         */
        Message msg = new Message();
        msg.arg1 = mResId;
        switch (statusCode) {
            case HttpStatus.SC_OK:
                Log.v(TAG, TAG + " parseErrorResponse json error ...");
                if (!mIsInBackground) {
                    ToastUtil.toast(R.string.data_error);
                }

                msg.what = R.id.doDataError;
                break;
            case HttpStatus.SC_NOT_FOUND:
                if (!mIsInBackground) {
                    ToastUtil.toast(R.string.request_url_not_found);
                }

                msg.what = R.id.BadOnline;
                break;
            case HttpStatus.SC_INTERNAL_SERVER_ERROR:
                if (!mIsInBackground) {
                    ToastUtil.toast(R.string.server_error);
                }

                msg.what = R.id.BadOnline;
                break;
            case HttpStatus.SC_REQUEST_TIMEOUT:
            default:
                if (!mIsInBackground) {
                    ToastUtil.toast(R.string.network_error);
                }

                msg.what = R.id.BadOnline;
                break;
        }
        mHandler.sendMessage(msg);
    }

}

