package com.common.framework.network.volley;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Created by guqingbo on 2016/4/13.
 * 根据afinal.http.AjaxParams
 * 去掉了MultipartEntity部分
 */
public class AjaxParams {
    private static final char[] MULTIPART_CHARS = "-_1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".toCharArray();
    private String boundary = null;
    private final static String ENCODING = "UTF-8";
    protected ConcurrentHashMap<String, String> urlParams = new ConcurrentHashMap<>();
    protected ConcurrentHashMap<String, FileWrapper> fileParams = new ConcurrentHashMap<>();
    ByteArrayOutputStream out;

    public AjaxParams() {
        this.init();
    }

    public AjaxParams(Map<String, String> source) {
        this.init();

        for (Map.Entry entry : source.entrySet()) {
            this.put((String) entry.getKey(), (String) entry.getValue());
        }

    }

    public AjaxParams(String key, String value) {
        this.init();
        this.put(key, value);
    }

    public AjaxParams(Object... keysAndValues) {
        this.init();
        int len = keysAndValues.length;
        if (len % 2 != 0) {
            throw new IllegalArgumentException("Supplied arguments must be even");
        } else {
            for (int i = 0; i < len; i += 2) {
                String key = String.valueOf(keysAndValues[i]);
                String val = String.valueOf(keysAndValues[i + 1]);
                this.put(key, val);
            }

        }
    }

    private void init() {
        generateBoundary();
    }

    /**
     * 随机生成Boundary
     */
    private void generateBoundary() {
        StringBuilder buf = new StringBuilder();
        Random rand = new Random();

        for (int i = 0; i < 30; ++i) {
            buf.append(MULTIPART_CHARS[rand.nextInt(MULTIPART_CHARS.length)]);
        }

        boundary = buf.toString();
    }

    public void put(String key, String value) {
        if (key != null && value != null) {
            this.urlParams.put(key, value);
        }

    }

    public void put(String key, File file) throws FileNotFoundException {
        this.put(key, new FileInputStream(file), file.getName());
    }

    public void put(String key, InputStream stream) {
        this.put(key, stream, null);
    }

    public void put(String key, InputStream stream, String fileName) {
        this.put(key, stream, fileName, null);
    }

    public void put(String key, InputStream stream, String fileName, String contentType) {
        if (key != null && stream != null) {
            this.fileParams.put(key, new AjaxParams.FileWrapper(stream, fileName, contentType));
        }

    }

    public void remove(String key) {
        this.urlParams.remove(key);
        this.fileParams.remove(key);
    }

    public String toString() {
        StringBuilder result = new StringBuilder();

        for (Map.Entry entry : urlParams.entrySet()) {
            if (result.length() > 0) {
                result.append("&");
            }

            result.append((String) entry.getKey());
            result.append("=");
            result.append((String) entry.getValue());
        }

        for (Map.Entry entry : fileParams.entrySet()) {
            if (result.length() > 0) {
                result.append("&");
            }

            result.append((String) entry.getKey());
            result.append("=");
            result.append(entry.getValue().toString());
        }

        return result.toString();
    }

    /**
     * 获取ContentType
     *
     * @return
     */
    public String getBodyContentType() {
        return ("multipart/form-data; boundary=" + this.boundary);
    }

    /**
     * 获取发送的body
     *
     * @return
     */
    public byte[] getBody() {
        byte[] body = null;
        if (!this.fileParams.isEmpty()) {
            out = new ByteArrayOutputStream();
            try{
                writeFirstBoundary();
                for (Map.Entry entry : this.urlParams.entrySet()) {
                    addUrlPart((String) entry.getKey(), (String) entry.getValue());
                }

                for (Map.Entry entry : fileParams.entrySet()) {
                    AjaxParams.FileWrapper file = (AjaxParams.FileWrapper) entry.getValue();
                    if (file.inputStream != null) {
                        addFilePart((String) entry.getKey(), file.getFileName(), file.inputStream, file.contentType);
                    }
                }
            }catch (Exception e){
                e.printStackTrace();
                out.reset();
            }finally {
                body = out.toByteArray();
                try {
                    out.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                out = null;
            }
        }
        return body;
    }

    public void addUrlPart(String key, String value) throws Exception{
            this.out.write(("Content-Disposition: form-data; name=\"" + key + "\"\r\n\r\n").getBytes(ENCODING));
            this.out.write(value.getBytes(ENCODING));
            writeBoundary();
    }

    public void addFilePart(String key, String fileName, InputStream fin, String type) throws Exception {
        try {
            type = "Content-Type: " + type + "\r\n";
            this.out.write(("Content-Disposition: form-data; name=\"" + key + "\"; filename=\"" + fileName + "\"\r\n").getBytes(ENCODING));
            this.out.write(type.getBytes(ENCODING));
            this.out.write("Content-Transfer-Encoding: binary\r\n\r\n".getBytes(ENCODING));
            byte[] e = new byte[4096];

            int flag;
            while ((flag = fin.read(e)) != -1) {
                this.out.write(e, 0, flag);
            }

            writeBoundary();

            this.out.flush();
        } catch (Exception e) {
            throw e;
        } finally {
            try {
                fin.close();
            } catch (IOException e) {
                e.printStackTrace();
            }

        }

    }

    public void writeFirstBoundary() throws Exception {
        out.write(("--" + this.boundary + "\r\n").getBytes(ENCODING));
    }

    public void writeBoundary() throws Exception {
        out.write(("\r\n--" + this.boundary + "\r\n").getBytes(ENCODING));
    }

    private static class FileWrapper {
        public InputStream inputStream;
        public String fileName;
        public String contentType;

        public FileWrapper(InputStream inputStream, String fileName, String contentType) {
            this.inputStream = inputStream;
            this.fileName = (fileName != null ? fileName : "nofilename");
            this.contentType = (contentType != null ? contentType : "application/octet-stream");
        }

        public String getFileName() {
            return this.fileName;
        }

        @Override
        public String toString() {
            return "contentType:" + contentType + ",filename:" + fileName;
        }
    }
}
